/* Written by James Henderson and Ivan Titov */
#ifndef _CLUSTER_H_

#define _CLUSTER_H_


/**
 * cluster structure definition
 */
struct cluster {
    /** child list and its size */
    int child_qty;
    /** I know, I know that  it should be children, not childs =) */
    struct cluster **childs;
    struct cluster *parent;
    
    /** cluster id (idx in clusters_arr for clustering) */
    int cluster_id;
    
    /** layer num - 0 for root of clustering */
    int layer_num;
    
};

struct clustering {
    /** hierarchy of clusters */
    struct cluster *root;
    
    /** all the clusters represented in one dimension array and there quantity */
    struct cluster **clusters_arr;
    /** quantity of clusters (all the nodes, not only leafes) */
    int cluster_qty;
    
};


/** struct for list handling */
struct _cl_list {
    struct _cl_list *next;
    struct _cl_list *prev;
    struct cluster *elem;
};



/** no attempt to make it work fast (done once on initalization) 
 *  @param parent the parent node to which to attach (if NULL - assuming creating root)
 *  @return created node
 * */
struct cluster *cl_create_child(struct cluster *parent);

/** method for creating clustering structure from cluster root */
struct clustering *cl_create_clustering(struct cluster *root);

/** freeing of all the clustering structure */
void cl_free(struct clustering *clustering);

/** debug cluster output */
void cl_print_cluster(struct cluster* c);

/** debug clustering output */
void cl_print_clustering(struct clustering* cluster_str);

/**
 * Read clusters from buffer (buffer contains the quantity of childs for clustering nodes in
 * width-first order
 * @param buffer buffer to read from
 * @param separs used separaters
 * @return the resulting clustering
 */
struct clustering  *cl_read_clustering(char* buffer, char* separs);

/**
 * Creates flat structure (root and childs)
 * @param child_qty root childs qty
 * @return resulting clustering
 */
struct clustering *cl_create_one_level(int child_qty);

/**
 * Determines the branching factor of the clustering
 * @param cluster_str the cluster structure to analyze
 * @return the max branching factor
 */
int cl_get_max_branch_fact(struct clustering *cluster_str);


/************************ private routines ***************************/
/** 
 * adds elem to the tail of the list
 * @param head head of the list
 * @param tail tail of the list
 * @param elem the elem of the node to add
 */
void _cl_add_to_tail(struct _cl_list **head, struct _cl_list **tail, struct cluster *elem);

/**
 * gets and removes element from the list head
 * @param head the list head
 * @param tail the list tail
 * @return cluster
 */
struct cluster* _cl_pop_from_head(struct _cl_list **head, struct _cl_list **tail);

/**
 * Creates new clustering based on given, extending the leafs which are marked in is_clusterid_prolonged with 1 adding 
 * subclusters (using given branch factor). 
 * @param is_clusterid_extended maps cluster_id to whether it has to be extended or not
 * @param branch_factor the branch factor for the clusters (nodes)  to be extended
 * @return the clustering 
 */
struct clustering *cl_extend_clustering(struct clustering* init_clustering, int is_clusterid_extended[], int branch_factor);

/**  
 * returns the specification which is the quantity of childs for the node in 
 * the traverse order: parent - first child  -... - last child 
 */ 
char *cl_get_specification(struct clustering *cluster_str);


/**
 * Removes the node from the clustering.
 * It is assumed that this node is leaf (if not error returned)
 * clusters_arr is not adjusted (NULL is placed), so it should be used carefully after (quick hack!)
 * @param cluster_str the clustering to deal with
 * @param cluster_id the cluster_id of the leaf to remove
 * @return negative if error, else if OK
 */
 
int cl_remove_leaf_wo_list_change(struct clustering *cluster_str, int cluster_id);

/**
 * Extends the cluster in a given subtree. 
 * @param new_cluster new cluster which subtree should be added (and extention of needed clusters should be performed in it)
 * @param old_cluster the cluster form which to copy
 * @param is_clusterid_extended maps cluster_id to whether it has to be extended or not
 * @param branch_factor the branch factor for the clusters (nodes)  to be extended 
 */
void _cl_extend_subtree(struct cluster *new_cluster, struct cluster *old_cluster, int is_clusterid_extended[], int branch_factor);

/**
 * frees the list (without freeing the elements)
 * @param head head  of the list
 */
void _cl_free_list(struct _cl_list *head);

    
/** internal freeing function */
void _cl_free(struct cluster* c);

/** 
 * enumerates the clusters 
 * @param cluster  cluster subclusters of which should be enumerated
 * @param pcluster_arr ptr to cluster array
 * @return new cluster array size
 */
int _cl_enumerate(struct cluster  *start_cluster, struct cluster ***pcluster_arr) ;

    
char *_cl_make_message(const char *fmt, ...);
    
void _cl_print_cluster(struct cluster* c, char* prefix);
void _cl_print_list(struct _cl_list *head, struct _cl_list *tail);


#endif

