/* code for changing tags used in gn*.c data */

#define DEBUG    0
#define DEBUG_ALLOC    0
#define DEBUG_INP 0
#define DEBUG_STRUCMOD 0
#define DEBUG_TRIG 0
#define DEBUG_READOUT 0
void bugbreak() {}

#include <stdlib.h>
/* #include "/usr/include/stdlib.h" ?*/
#include <stdio.h>
#include <math.h>
#include <string.h>

#define MAXLABEL 100  /* max length of label string */
#define MAXDATALINE 10000  /* max length of one line of words */
#define MAXSENTLENGTH 1000
#define MAXWORDLENGTH 100 

#define UNTRIGGER "***"
#define NONTRIGGER "_"



/** TAG, LABEL, AND WORD CODES **/

int tag_id(char *str)
{
  if (!strcmp(str, "#"))
    return(0);
  else if (!strcmp(str, "$"))
    return(1);
  else if (!strcmp(str, "''"))
    return(2);
  else if (!strcmp(str, ","))
    return(3);
  else if (!strcmp(str, "-LRB-"))
    return(4);
  else if (!strcmp(str, "-RRB-"))
    return(5);
  else if (!strcmp(str, "."))
    return(6);
  else if (!strcmp(str, ":"))
    return(7);
  else if (!strcmp(str, "^^^"))
    return(8);
  else if (!strcmp(str, "``"))
    return(9);
  else if (!strcmp(str, "CC"))
    return(10);
  else if (!strcmp(str, "CD"))
    return(11);
  else if (!strcmp(str, "DT"))
    return(12);
  else if (!strcmp(str, "EX"))
    return(13);
  else if (!strcmp(str, "FW"))
    return(14);
  else if (!strcmp(str, "IN"))
    return(15);
  else if (!strcmp(str, "JJ"))
    return(16);
  else if (!strcmp(str, "JJR"))
    return(17);
  else if (!strcmp(str, "JJS"))
    return(18);
  else if (!strcmp(str, "LS"))
    return(19);
  else if (!strcmp(str, "MD"))
    return(20);
  else if (!strcmp(str, "NN"))
    return(21);
  else if (!strcmp(str, "NNP"))
    return(22);
  else if (!strcmp(str, "NNPS"))
    return(23);
  else if (!strcmp(str, "NNS"))
    return(24);
  else if (!strcmp(str, "PDT"))
    return(25);
  else if (!strcmp(str, "POS"))
    return(26);
  else if (!strcmp(str, "PRP"))
    return(27);
  else if (!strcmp(str, "PRP$"))
    return(28);
  else if (!strcmp(str, "RB"))
    return(29);
  else if (!strcmp(str, "RBR"))
    return(30);
  else if (!strcmp(str, "RBS"))
    return(31);
  else if (!strcmp(str, "RP"))
    return(32);
  else if (!strcmp(str, "SYM"))
    return(33);
  else if (!strcmp(str, "TO"))
    return(34);
  else if (!strcmp(str, "UH"))
    return(35);
  else if (!strcmp(str, "VB"))
    return(36);
  else if (!strcmp(str, "VBD"))
    return(37);
  else if (!strcmp(str, "VBG"))
    return(38);
  else if (!strcmp(str, "VBN"))
    return(39);
  else if (!strcmp(str, "VBP"))
    return(40);
  else if (!strcmp(str, "VBZ"))
    return(41);
  else if (!strcmp(str, "WDT"))
    return(42);
  else if (!strcmp(str, "WP"))
    return(43);
  else if (!strcmp(str, "WP$"))
    return(44);
  else if (!strcmp(str, "WRB"))
    return(45);
  else
    {
      fprintf(stderr, "Error: unrecognized tag %s\n", str);
      return(99);
    }
}


/** DATATYPE DEFINITIONS **/



/** GLOBAL VARIABLES **/

int sentence_number;

/** PROCEDURE DECLARATIONS **/

int main(int , char *[]);
void change_tags(FILE *, FILE *);

void usage_error(char *);
void error(char *);



/** CODE **/

/* Parse the command line and do the right thing. */
int main(int argc, char *argv[])
{
  int nextarg;
  char data_filename[FILENAME_MAX];
  char tags_filename[FILENAME_MAX];
  FILE *data;
  FILE *tags;

  int errno = 0;
  nextarg = 1;

  if (argc < 2)  
    usage_error(argv[0]);

  if (strcmp(argv[nextarg], "-h") == 0)
    usage_error(argv[0]);

  strcpy(data_filename, argv[nextarg]);
  nextarg++;
  if ((data=fopen(data_filename, "r")) == NULL)
    error("Error: can't open data file.");
  strcpy(tags_filename, argv[nextarg]);
  nextarg++;
  if ((tags=fopen(tags_filename, "r")) == NULL)
    error("Error: can't open taged word file.");

  change_tags(data, tags);
  fclose(data);

  return(errno);
}


void change_tags(FILE *data, FILE *tags)
{
  int i;
  char buffer[MAXDATALINE];
  char *inpp;
  char tags_buffer[MAXDATALINE];
  char *tags_tokp;
  int position;
  int line; 
  
#if DEBUG_INP
  printf("\n");
#endif

  if (fgets(tags_buffer, MAXDATALINE, tags) == NULL)
    error("no lines in tagged word file");
  strtok(tags_buffer, "_");  /* initialize strtok() on tag_buffer */

  line = -1;
  while (fgets(buffer, MAXDATALINE, data) != NULL)
    {
      line++;
      inpp = buffer;
      while (inpp[0] != '\n')
	{
	  if (inpp >= buffer+MAXDATALINE)
	    error("Error: no newline found or line too long.");

	  if (inpp[0] != '>')
	    {
	      printf("%c", inpp[0]);
	      inpp++;
	    }
	  else  /* inpp[0] == '>' */
	    {
	      printf("%c", inpp[0]);
	      inpp++;
	      inpp++;  /* skip space */
	      while (inpp[0] != ' ' && inpp[0] != '-' && inpp[0] != '\n')
		inpp++;  /* skip old tag */
	      /* find next new tag */
	      while ((tags_tokp = strtok(NULL, " \n")) == NULL)
		{ 
		  if (fgets(tags_buffer, MAXDATALINE, tags) == NULL)
		    error("too few lines in tagged word file");
		  strtok(tags_buffer, "_");
		}
	      strtok(NULL, "_");  /* skip word in tag file */
	      /* output new tag */
	      printf(" %d", tag_id(tags_tokp));
	    }
	}
      printf("\n");
    }
}
	  




void usage_error(char *name)
{
  printf("Usage:\n  %s -h | datafile taggedfile\n", name);  
  exit(0);
}

void error(char *mes)
{
  printf("\n%s\n", mes); 
  exit(0);
}

