/* read in reverse sorted lines of "tag_id freq word" and output the code
   for converting word strings to tag_id,word_id_list lists to stdout.  Also
   adds tag_id,word_id_list lists for UNKNOWN for every bin and tag_id.  If
   an UNKNOWN case has a lower frequency than the threshold, then add the
   lowest frequency word above the threshold to the UNKNOWN case.  */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

#define MAXTAGS 100
#define MAXLINE 1000


/* Function Declaration */
void print_word_case(char *, int );


int main(int argc, char *argv[])
{
  /*   FILE *infp; */
  char buffer[MAXLINE];
  int i, j, k, m, b;
  int holdtag;
  int holdfreq;
  char holdword[MAXLINE];
  int prevtag;
  int prevfreq;
  char prevword[MAXLINE];
  int nexttag;
  int nextfreq;
  char *nextword;
  int id;
  int maxtagoutputs;
  int unkn_freq;

  int numtags;
  int threshold;
  int unkn_threshold;
  int tag_unkn[MAXTAGS];

  if (argc < 2 || argc > 3)
    {
      printf("Usage:  %s threshold [ unknown_threshold ]\n  old data to stdin and new to stdout.\n", 
	     argv[0]);
      exit(0);
    }

  threshold = atoi(argv[1]);
  if (argc > 2)
    unkn_threshold = atoi(argv[2]);
  else
    unkn_threshold = threshold;

  printf("/* threshold = %d, unknown_threshold = %d */\n\n", 
	 threshold, unkn_threshold);
  printf("#include <stdlib.h>\n#include <stdio.h>\n#include <string.h>\n\n");
  printf("char *word_id_string(int tag, char *word)\n{\n  char *res;\n\n");

  maxtagoutputs = 0;
  numtags = 0;

  if (fgets(buffer, 1000, stdin) != NULL)  {
    prevtag = atoi(strtok(buffer, " \t\n"));
    prevfreq = atoi(strtok(NULL, " \t\n"));
    nextword = strtok(NULL, " \t\n");
    strcpy(prevword, nextword);

    /* first tag is the largest tag */
    numtags = prevtag + 1;
    printf("  ");

    id = 0;
    unkn_freq = 0;
    holdtag = -1;
    printf("if (tag == %d)\n    {\n      ", prevtag);

    while (1)
      {
	if (fgets(buffer, 1000, stdin) != NULL)
	  {
	    nexttag = atoi(strtok(buffer, " \t\n"));
	    nextfreq = atoi(strtok(NULL, " \t\n"));
	    nextword = strtok(NULL, " \t\n");
	  }
	else
	  {
	    nexttag = -1; /* flag for end case */
	  }

	if (nexttag == prevtag) 
	  {
	    if (prevfreq >= threshold) /* otherwise word too infrequent */
	      {
		if (holdtag >= 0)
		  {
		    print_word_case(holdword, id);
		    id++;
		  }
		holdtag = prevtag;
		holdfreq = prevfreq;
		strcpy(holdword, prevword);
	      }
	    else
	      {
		unkn_freq += prevfreq;
	      }
	  }
	else  
	  {
	    /* end of prevtag entry */

	    /* last word always in UNKNOWN, since UNKNOWN can't be empty */
	    unkn_freq += prevfreq; 
	    if (unkn_freq >= unkn_threshold)
	      {
		if (holdtag >= 0)
		  {
		    print_word_case(holdword, id);
		    id++;
		  }
	      }
	    else if (holdtag >= 0) 
	      {
		printf("\n        /* word \"%s\" (freq=%d) added to UNKNOWN (freq=%d) case for tag %d */", 
		       holdword, holdfreq, unkn_freq, holdtag);
	      }
	    else
	      {
		printf("\n        /** warning: tag %d has frequency %d, below unknown_threshold %d **/",
		       prevtag, unkn_freq, unkn_threshold);
	      }
	    if (prevfreq >= threshold)
	      {
		printf("\n        /* word \"%s\" (freq=%d) added to UNKNOWN (freq=%d) case for tag %d */", 
		       prevword, prevfreq, unkn_freq - prevfreq, prevtag);
	      }

	    /* unknown word case for previous tag */
	    tag_unkn[prevtag] = id;
	    printf("\n        {\n          res = \"");
	    printf(",%d", tag_unkn[prevtag]); 
	    printf("\";\n        }");
	    printf("\n    }\n    /* tag %d has %d outputs",
		   prevtag, tag_unkn[prevtag] + 1);
	    printf("*/");
	    printf("\n  else ");
	    if (tag_unkn[prevtag] + 1 > maxtagoutputs)
	      maxtagoutputs = tag_unkn[prevtag] + 1;

	    /* start of nexttag entry */
	    id = 0;
	    unkn_freq = 0;
	    holdtag = -1;
	    if (nexttag >= 0)
	      printf("if (tag == %d)\n    {\n      ", nexttag);
	  }

	if (nexttag >= 0)
	  {
	    prevtag = nexttag;
	    prevfreq = nextfreq;
	    strcpy(prevword, nextword);
	  }
	else
	  break;  /* loop exit */
      }
  }

  /* BADTAG case */
  printf("\n    {\n      res = \"");
  printf(",%d", -1);
  printf("\";\n    }\n\n  return(res);\n}\n");

  /* output further information in comments */
  printf("\n/* max outputs per tag:  %d */", maxtagoutputs); 

  id = 0;
  for (prevtag = 0; prevtag < numtags; prevtag++)
    id += tag_unkn[prevtag] + 1;
  printf("\n/* number of word outputs:  %d */", id);

  printf("\n/* number of tags and number of word outputs for each tag, for .inf files:");
  printf("\n%d\n", numtags);
  for (prevtag = 0; prevtag < numtags; prevtag++)
    {
      printf("%d ", tag_unkn[prevtag] + 1);
    }      
  printf("\n*/\n\n");

}


void print_word_case(char *word, int id) 
{
  char *wp;

  printf("if (!strcmp(word, \"");
  for (wp = word; *wp != '\0'; wp++)
    {
      if (*wp == '\\')
	printf("\\\\");
      else
	printf("%c", *wp);
    }
  printf("\"))\n        {\n          res = \"");
  printf(",%d", id);
  printf("\";\n        }\n      else ");
}


