/* Written by James Henderson and Ivan Titov */
#include "list.h"
#include <malloc.h>
#include <assert.h>

#define CHECK_PTR(x) if ((x) == NULL) { printf("Can't allocate memory block\n"); exit(-1);}

/** 
 * adds elem to the tail of the list
 * @param head head of the list
 * @param tail tail of the list
 * @param elem the elem of the node to add
 */
void add_to_tail(struct list **head, struct list **tail,void *elem) {

    struct list *new_node; 
   
    assert((*head == NULL && *tail == NULL) || (*head != NULL && *tail != NULL));

    new_node = (struct list*) malloc(sizeof(struct list));
    CHECK_PTR(new_node);
    new_node->elem = elem;
    new_node->next = NULL;
    new_node->prev = NULL;
    
    
    if (*head == NULL) {
        /** no list yet */
        *tail = *head = new_node;
    } else {
        (*tail)->next = new_node;
        new_node->prev = (*tail);
        *tail = new_node;
    }
}


/** 
 * adds elem to the head of the list
 * @param head head of the list
 * @param tail tail of the list
 * @param elem the elem of the node to add
 */
void add_to_head(struct list **head, struct list **tail,void *elem) {

    struct list *new_node; 
   
    assert((*head == NULL && *tail == NULL) || (*head != NULL && *tail != NULL));

    new_node = (struct list*) malloc(sizeof(struct list));
    CHECK_PTR(new_node);
    new_node->elem = elem;
    new_node->next = NULL;
    new_node->prev = NULL;
    
    
    if (*head == NULL) {
        /** no list yet */
        *tail = *head = new_node;
    } else {
        (*head)->prev = new_node;
        new_node->next = (*head);
        *head = new_node;
    }
}

/**
 * gets and removes element from the list head
 * @param head the list head
 * @param tail the list tail
 * @return void
 */
void* pop_from_head(struct list **head, struct list **tail) {
    void *c;
    
    if (*head == NULL || *tail == NULL) {
        return NULL;
    }
   
    c = (*head)->elem;
    
    //last elem in list
    if (*head == *tail) {
        free(*head);
        *head = NULL;
        *tail = NULL;
    } else {
        *head = (*head)->next;
        free((*head)->prev);
        (*head)->prev = NULL;
    }

    return c;
}

/**
 * gets and removes element from the list tail
 * @param head the list head
 * @param tail the list tail
 * @return void
 */
void* pop_from_tail(struct list **head, struct list **tail) {
    void *c;
    
    if (*head == NULL || *tail == NULL) {
        return NULL;
    }
   
    c = (*tail)->elem;
    
    //last elem in list
    if (*head == *tail) {
        free(*head);
        *head = NULL;
        *tail = NULL;
    } else {
        *head = (*tail)->prev;
        free((*tail)->next);
        (*tail)->next = NULL;
    }

    return c;
}

/**
 * frees the list (without freeing the elements)
 * @param head head  of the list
 * @param tail tail of the list
 */
void free_list(struct list **head, struct list **tail) {
    
    struct list *swap;

    while((*head) != NULL) {
        swap = *head;
        *head = (*head)->next;
        free(swap);
    }
    *head = NULL;
    *tail = NULL;
}

