/* Written by James Henderson and Ivan Titov */
#ifndef _SSN_COMMON_API_H_
#define _SSN_COMMON_API_H_

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <stdarg.h>
#include <values.h>
#include "ssn_common_types.h"

/** @ivan ssn_ prefix has the name the functions wich I call externally (E.g. from SVMLight) , I use this as the name space */
/** as the result it should be separated on .h file used internally (if needed) and external .h (external interface) */

/** set all the file names on the basis of given names: first is stem, other two includes suffix */
void set_all_file_names(char *file_name_stem, char *data_name, char *val_name);

/** returns the name of the weight file */
char *get_weight_file();

/** set the prediction beam */
void set_pred_beam(int _predbeam);

/** sets the phase beam */
void set_phase_beam(int _phasebeam);

/** sets whethe joint error should be used  or not */
void set_joint_error_flag(int flag);


/* input/target setting */
int comp_sentence(FILE *);
int load_period(char *, int);
char *first_token(char *);
char *next_token();
void pop_stack(struct stack *);
int relative_pop_stack(int);
void push_stack(struct stack *, int);
void empty_stack(struct stack *);
int realloc_stack_content(struct stack *, int);
void copy_stack(struct stack *, struct stack *);
void comp_period_training_act(int);
void comp_state_act(struct option *);
/* network computation */
void comp_hidrho_option_act(struct option *);
void comp_output_act_range(struct option *, int, int);
void normalize_output_act_range(struct option *, int, int, int);
int in_range(int, int, int, int);
/* training */
void learning(int);
void validation_learning(int, int);
void training_epoch();
int opt_comp_sentence_act(FILE *, FILE *);
void opt_comp_sentence_del(int);
void comp_sentence_alternatives(struct option_stack *,
                                struct option_stack *, int);
void comp_default_output_deltas(struct option *);
void zero_output_deltas(struct option *);
double tail_estimate(double, struct option **, int);
void opt_comp_hidden_del(struct option *);
void comp_prehidtau_del(int, int);
void update_weights(int);
void scatter_weights(double);
/* testing */
void evaluation();
void testing_epoch();
int test_sentence(FILE *, FILE *);
void search_options(int, struct option_stack *, double, int);
struct option_stack *generated_options(struct option *, int);
void punt_queue(struct option_stack *);
int worth_pursuing(struct option *);
void init_phases_for_options();
struct option *root_prediction();
void pluck_best_option(int, struct option **);
void init_option_phases(struct option *);
void comp_option_states(struct option *);
void set_option_states(struct option *);
int allocate_option_phase(int, int, int, int, int);
void init_free_phases();
int expand_free_phases(int, int);
struct tree *perform_derivation(struct option *);
struct tree_list *perform_options(struct tree_list *, int, struct option *,
                                  int);
int read_idtext(FILE *);
int read_tagword_spec(char *, int);
void set_end_input(int);
int read_candidate_derivations(FILE *, int, int);
int read_target_derivation(FILE *);
void comp_option_list_act(struct option_stack *, struct option_stack *);
struct tree *read_target(FILE *, int);
struct tree_list *start_root(int, int, struct tree_list *);
void expand_conflations(struct tree *);
struct option *create_option(int, int, double, double, int, int,
                             struct option *, int, int, int, int, int,
                             struct option *, struct option *,
                             struct option *, struct option *,
                             int, int, int, int);
void set_option(struct option *, int, int, double, double,
                int, int, struct option *, int, int, int, int, int,
                struct option *, struct option *, struct option *,
                struct option *, int, int, int, int);
struct option *create_null_option();
void free_option_branch(struct option *);
void free_option(struct option *);
void free_option_only(struct option *);
void free_option_descendents(struct option *);
void free_next_options(struct option *);
struct option_stack *mergesort_options(struct option_stack *,
                                       struct option_stack *);
void display_option(struct option *, int);
void display_symbolic_option(struct option *);
struct option_stack *create_option_stack(struct option *,
                                         struct option_stack *,
                                         struct option_stack *);
struct option_stack *create_option_cons(struct option *);
struct option_stack *popfree_option_stack(struct option_stack *);
void pluckfree_option_stack(struct option_stack *);
struct option_stack *incr_option_stack_next(struct option_stack *);
struct
option_stack_stack *create_option_stack_stack(struct option_stack *,
                                              struct option_stack_stack *,
                                              struct option_stack_stack *);
struct option_stack_stack *create_option_stack_cons(struct option_stack *);
struct
option_stack_stack *popfree_option_stack_stack(struct option_stack_stack
                                               *);
void opt_comp_err_summary(struct option *);
void comp_structure_summary(int);
void test_constituents(int *, int *, int *, int);
struct tree_list *flatten_tree(struct tree *, struct tree_list *);
void update_recprec();
void query_epoch();
void init_query();
void comp_query(int);
void print_query();
void rerun_testing_sentence(int);
struct tree *create_tree();
void free_tree(struct tree *);
void free_all_tree(struct tree *);
struct tree_list *create_tree_list(struct tree *, struct tree_list *);
void free_tree_list(struct tree_list *);
void free_all_tree_list(struct tree_list *);
void pluckfree_tree_list(struct tree_list *);
struct tree_list *popfree_tree_list(struct tree_list *);
/* file I/O */
void write_testing(int);
void read_last_weights();
double read_weights(char *wgtfile);
void save_weights(char *, double);
void initialize();
void read_inf_changes();
void copy_weights(char *, char *);
void copyfile(char *, char *);
void write_derivations(struct option *, FILE *);
void write_option_spec(struct option *, FILE *);
void write_tree(struct tree *, char *, char *);
int write_subtree(struct tree *, int, FILE *);
/* space allocation */
void expand_periods();
int expand_phases(int);
int expand_outputs();
/* debugging outputs */
void display_tree(struct tree *);
int display_subtree(struct tree *, char *, int);
void display_epoch();     /** not updated **/
void display_net();
void display_link_deltas();
void check_output_act_del(int, int);
void check_output_act_del_range(int, int, struct output *, int, int,
                                int, int, int, int);
void display_deltas(int);
void display_total_deltas(int);
void display_state(int);
void display_target_output(int);
double display_symbolic_target_output(int, double);
void display_states(int);
void display_opt_stack(struct option *);
void display_stack(struct stack *);
void display_consts(int);
void display_target_parchild(int);      /** not updated **/
void display_parchild(int);      /** not updated **/
void display_needed(int);       /** not updated **/
double print_stat(double ***, int, int, int);        /** not updated **/
void display_choice_errors(int);      /** not updated **/
int number_choice_errors(int);      /** not updated **/
/* decoding inputs and labels */
int tag_id(char *);
int label_id(char *);
char *id_tag(int);
char *id_label(int);
char *id_word(int);
void print_input(int);
/* error outputs */
void error(char *);
void usage_message(char *);
void usage_error(char *);
void usage_help(char *);
void warning(char *);
void fill_input_clusterized(struct input *, int, char *, char *);
void init_best_options(int);





/** Macro definitions **/

#define RAND_WEIGHT (RAND_RANGE*2.0*drand48()-RAND_RANGE)
#define RAND_WEIGHT_OUT (eta*(RAND_RANGE*2.0*drand48()-RAND_RANGE))
/* condition for output units which are needed even when phase!=period */
#define RHO_OUTPUT(u) 1

/* define mapping to/from output unit numbers.  
     Format: [ structure projlabel (hdprlabel) predtag (predword)] */
#define projlabel_to_output(id)  (labeloutputs + id)
#if HEADOUTS
#define hdprlabel_to_output(id)  (hdlabeloutputs + id)
#endif
#define predtag_to_output(id)    (tagoutputs + id)
/* maps from word id to memory space */
#define predword_to_output(id)   (wordoutputs + id)
#define output_to_projlabel(ou)  (ou - labeloutputs)
#if HEADOUTS
#define output_to_hdprlabel(ou)  (ou - hdlabeloutputs)
#endif
#define output_to_predtag(ou)    (ou - tagoutputs)

#define CHILDID_TO_MEMOUT(child_id) ((child_id) + wordoutputs)

#define CLUSTERID_TO_REALOUT(cluster_id, out1) (wordoutputs + out1_first_word[(out1)] + (cluster_id))

#define CLUSTERID_TO_WORDID(cluster_id, out1) ((cluster_id) + out1_first_word[(out1)])

#define WORDID_TO_CHILDID(wordid, out1, first_child_cluster_id) ((wordid) - out1_first_word[(out1)] - (first_child_cluster_id))


#define WORDID_TO_MEMOUT(wordid, memoffset) ((wordid) - (memoffset) + wordoutputs)
#define REALOUT_TO_WORDID(realout) ((realout) - wordoutputs)

#define IS_CLUSTER_OPT(opt) ((opt)->dec_cluster != NULL)
#define IS_CLUSTER_MEMOUT(memout) ((memout) >= wordoutputs)

#define IS_OPT_COMPUTED(opt)  ((opt)->firstout <= (opt)->lastout)
#endif
