/* Written by James Henderson and Ivan Titov */
#ifndef _SSN_COMMON_CONSTS_H_
#define _SSN_COMMON_CONSTS_H_


/* debugging flags */
#define ERRSUMMARY 1
#define CHECKACTDEL 0
#define CHECKOVERFLOW 0
#define RECHECKTEST 0
#define CHECKGARBAGE 0
#define SAFENEXTOPTIONS 0
#define DISPITER 1
#define DISPINF 0
#define DISPMEM 1
#define DISPSEN 0               /* =2 for short format */
#define FILEPROG 1
#define FILEWARN 1
#define FILEDISP 1
#define PRINTDIST 0
#define NOFREEING 0

/* Discriminative model requires:
   TAUHIDDEN=NOPREDICT=TAULOOKAHEAD=TAUBACKWARD=1 */

#define HEADOUTS     0          /* include head specification in outputs and unheaded
                                   input.  */
#define HEADHIDDEN   0          /* include head child hidden reps in context.  Use
                                   with HEADOUTS=1.  */
#define HEADWORDS    0          /* include head word in inputs.  Use with
                                   HEADOUTS=1.  */
#define NOPREDICT    0          /* use a discriminative model, without predictions of
                                   tag-word pairs.  Use with PREDICTALL=0. Best to
                                   use it with TAULOOKAHEAD=1 and/or
                                   TAUHIDDEN=1,TAUBACKWARD=1.  */
#define CHILDCONTEXT 1          /* provide disambiguated most recent child
                                   label/state in rho context.  */
#define CHOPCONTEXT  0          /* provide disambiguated project/attach/modify
                                   operation with which most recent child ended in
                                   rho context.  */
#define LABELTRIGPARENTINP 0    /* use labels instead of hidden representations
                                   when inputting info about triggering parent 
                                   nonterminal.  */
#define LABELTRIGCHILDINP 0     /* use labels instead of hidden representations
                                   when inputting info about triggering child
                                   nonterminals.  */
#define LABELHEADCHILDINP 0     /* use labels instead of hidden representations
                                   when inputting info about head child
                                   nonterminals.  */
#define LABELCHILDINP 0         /* use labels instead of hidden representations when
                                   inputting info about previous child nonterminals. */
#define LABELCONTEXTINP 0       /* use labels (already input) instead of hidden
                                   representations when inputting info about
                                   previous state of top nonterminals. */
/* Conditional compilation which does not affect weightfile format,
   but does affect weightfile meaning.  1=yes, 0=no. */
#define ROOTTAGCOND  0          /* zero output for predicting tag ^^^ */
#define ROOTLABELCOND 0         /* zero output for projecting label "" */
#define ONLYPREDCONDDEL 1       /* Set the conditionalizing component of the output
                                   delta to zero for all non-predictions. */
#define PLUGLEAKS    1          /* estimate the backward probabilities for pruned
                                   options.  */
#define LOCALESTIMATE 1         /* estimate the backward probabilities for pruned
                                   options using the predictions for options
                                   continueing from the pruned option's previous
                                   option.  Use with PLUGLEAKS=1,USEINCOMPLETE=0. */
#define LOSTTAIL     0          /* use lost_probability as tail probability.  Use
                                   with PLUGLEAKS=0  */
#define UNHEADEDCOND 0          /* conditionalize probabilistic interpretation on
                                   being unheaded, through set outputs (only
                                   prediction possible). */
#define HEADEDPARENTCOND 0      /* conditionalize probabilistic interpretation on
                                   triggering parent being headed, through zeroed
                                   outputs (head attachment not possible). */
#define TAULOOKAHEAD 0          /* use input at now+1 for tau input.  Use with
                                   NOPREDICT=1.  */
#define TAUBACKWARD  0          /* use the tau hidden layer to compute unbounded
                                   lookahead.  Use with TAUHIDDEN=1. */
#define TANHHIDDEN   1          /* use the tanh activation function for hidden (and
                                   combination) units. */

/* conditional compilation which does not affect weightfile meaning.
   1=yes, 0=no. */
#define LOOKAHEADPRUNING 1      /* allow simulation of lookahead by pruning past
                                   best options based on sum over current best
                                   options.  */
#define TRAININITLESS 1         /* use TRAININITLESSRATE*eta for initstate updates */
#define TRAININITLESSRATE 0.2   /* use with TRAININITLESS=1 */
#define INITOUTPUTBIAS 0        /* initialize output biases to their unigram
                                   relative frequencies, read from .inf file.  */
#define NOTRAINUNKNOWN 0        /* don't use error from unknown words for anything
                                   but updating their bias. */
                         /** some improvement on deb_training_v4_200 **/
#define NOTRAINUNKNOWNHID 0     /* don't pass error from unknown words to the
                                   hidden layer. */
#define DEFAULTUNKNOWN 0        /* treat all non-bias weights to unknown word
                                   outputs as always 0  */
#define REGULARIZE   1          /* apply weight decay in weight updates? */
#define ZEROWORDINP  0          /* initialize weights from word inputs to zero. */
#define ERRVALIDATE  0          /* use the probability of the average output as the
                                   cross validation score */
#define FRZHIDOUT    0          /* don't train links from hidden layer to outputs */
#define FRZWRDOUT    0          /* don't train links from word inputs to outputs */
#define FRZBIASOUT   0          /* don't train biases for outputs */
#define FRZHIDHID    0          /* don't train links from hidden context to hidden
                                   layer */
#define FRZCONTHID   0          /* don't train links from disambiguated output
                                   context to hidden layer */
#define FRZNONWRDHID 0          /* don't train links from non-word inputs to hidden
                                   layer */
#define FRZWRDHID    0          /* don't train links from word inputs to hidden layer */
#define FRZBIASHID   0          /* don't train biases for hidden layer */

/* size steps when allocating space for longer sequences.  Must be greater
   than 2.  */
#define ALLOCSIZE 20

/* define some testing parameters */
#define MAXPREDBEAM 100         /* max number of prediction options pursued for a
                                   given word. */
#define MAXPHASEBEAM 20         /* max number of options added to the chart for any
                                   given previous option's phase, unless a
                                   prediction.  (>48 is no limit?)  */
#define MAXCANDIDATES MAXPREDBEAM       /* max number candidate derivations read */
#define MAXPHASES 30000         /* max number of phases which are allocated */
#define MAXPROJDEPTH 3          /* max depth of a sequence of projections */
#define MAXGARBAGEOPTS 10000    /* List of garbage options with state allocated
                                   is freed after sentence when this limit
                                   reached.  Use with GARBAGEOPTS. */
#define TAIL_DEFAULT_EXP -3.126627      /* ln of probability per word of
                                           incorrect cases when no search
                                           for incorrect cases is done. */
#define TAIL_LENGTH_EXP 0.05518
/* #define TAIL_BEAM_EXP   -0.065  * estimated with sums */
#define TAIL_BEAM_EXP   -0.023765       /* estimated with integral */
#define TAIL_AVE_BEST_EXP -3.181807

/* define some learning parameters */
#define RAND_RANGE 0.05         /* +/- range of random weight seeds */
#define STOPAFTER 4             /* don't terminate before this many suboptimal
                                   validations, ending in a drecrease */
#define MAXRED 0.001            /* eta never reduced more than this amount */
#define REDCRIT 0.0             /* reduce when error above last error plus this
                                   times difference between training error after
                                   weightfile and training error before/during
                                   weightfile. */

/* define size of each layer */
#define MAXNUMRHO 80            /* max for (rho) hidden layer size   <<<< */
#define MAXNUMTAU 100           /* max for tau hidden (mostly obsolete) */
#define MAXNUMINP1 46           /* max number of preterminal tags  <<<< */

#define MAXCLUSTERLEVELS 10

#if ! NOPREDICT
#define MAXNUMOUT1 MAXNUMINP1   /* max number of tag outputs */
#else
#define MAXNUMOUT1 0
#endif
#if PREBINDATA
#define MAXNUMBINS 1
#else
#define MAXNUMBINS 1            /* Max Word Bin: max number of the word bin */
#endif


#define MAXNUMINP2 4215        /* max number of word inputs  <<<< */

#if ! NOPREDICT
#define MAXNUMOUT2 4215        /* max number of word outputs  <<<< */
#else
#define MAXNUMOUT2 0
#endif


/* output layer's structure is:  structure operations (NUMSTRUC), predicted
   tags (numout1), projected labels (numlabels), [projected head labels
   (numlabels)]  */
#if HEADOUTS
#define NUMSTRUC 6              /* number of structure outputs.  must start at 0 */
#define NUMTRIG 4               /* number of outputs that trigger new phases,
                                   including preterms.  matrix root must be -1 and
                                   rest must be first in list: */
#define NUMCHOP 5               /* number of outputs with which a child can attach to 
                                   its parent.  Must be last NUMCHOP of NUMSTRUC
                                   outputs. */
#define ROOTFLAG -1             /* the matrix root */
#define PREDOUT 0               /* predict (>) output unit */
#define PROJOUT 1               /* project (|) output unit */
#define HDPROUT 2               /* head project (|h) output unit */
#define ATTOUT  3               /* attach (^) output unit */
#define HDATOUT 4               /* head attach (^h) output unit */
#define MODOUT  5               /* modify (+) output unit */
#else
#define NUMSTRUC 4              /* number of structure outputs */
#define NUMTRIG 3               /* number of outputs that trigger new phases,
                                   including preterms.  matrix root must be -1 and
                                   rest must be first in list: */
#define NUMCHOP 3               /* number of outputs with which a child can attach to 
                                   its parent.  Must be last NUMCHOP of NUMSTRUC
                                   outputs. */
#define HDPROUT -3              /* no head project (|h) output unit */
#define HDATOUT -3              /* no head attach (^h) output unit */
#define ROOTFLAG -1             /* the matrix root */
#define PREDOUT 0               /* predict (>) output unit */
#define PROJOUT 1               /* project (|) output unit */
#define ATTOUT  2               /* attach (^) output unit */
#define MODOUT  3               /* modify (+) output unit */
#endif
/* range of pre-prediction outputs */
#define PREPREDFIRST  PREDOUT
#define PREPREDLAST   MODOUT
/* See also macro definitions below */

/* define sizes for various output arrays */
#define MAXNUMLAB 33            /* max number of nonterminal labels  <<<< */
#if HEADOUTS
#define MAXNUMTARMEM (NUMSTRUC+MAXNUMLAB+MAXNUMLAB+MAXNUMOUT1)
#else
#define MAXNUMTARMEM (NUMSTRUC+MAXNUMLAB+MAXNUMOUT1)
#endif
/* max for the memory locations for target values,
   which does not include explicit word predicitons.*/
#define MAXNUMOUT (MAXNUMTARMEM+MAXNUMOUT2)
/** maximal number of prediction options for option */
#define MAXWORDOUTS 1070        /* max outputs per tag  <<<< */
#define MAXNUMOUTMEM (MAXNUMTARMEM+MAXWORDOUTS)
/* max for the memory locations for output values. */

/* define sizes for various input arrays */
#define MAXDATALINE 2000        /* max number of characters in a line of */
/* training data */
#define MAXINFLINE 50000        /* max number of characters in a line of a .inf
                                   file */
#define MAXWGTLINE 350000       /* max number of characters in a line of a weight
                                   file (14 * max non-input layer size +) */
#define MAXWORDLENGTH 100       /* max length of the text for a word. */
#define MAXLABELLENGTH 100      /* max length of the text for a label. */
#define FILENAMESIZE 300        /* max length of the text for a filename. */


/** discriminative reranking using fisher score (voted perceptron is used) 
 * CANDIDATES = 1 should be set
 * */

#endif
