/* Written by James Henderson and Ivan Titov */
#ifndef _SSN_COMMON_GLOBALS_H_
#define _SSN_COMMON_GLOBALS_H_

#include "ssn_common_consts.h"

/** Global Data Structures **/

/* Declare some global string variables */
extern char filename[FILENAMESIZE];
extern char inffile[FILENAMESIZE];
extern char weightfile[FILENAMESIZE];
extern char newweightfile[FILENAMESIZE];
extern char datafile[FILENAMESIZE];
extern char outfile[FILENAMESIZE];
extern char brafile[FILENAMESIZE];
extern char gldfile[FILENAMESIZE];
extern char resfile[FILENAMESIZE];
extern char topfile[FILENAMESIZE];
extern char candfile[FILENAMESIZE];
extern char traincandfile[FILENAMESIZE];
extern char valdatafile[FILENAMESIZE];
extern char valweightfile[FILENAMESIZE];
extern char retestfile[FILENAMESIZE];
extern char testingfile[FILENAMESIZE];
extern char neededfile[FILENAMESIZE];
extern char valneededfile[FILENAMESIZE];
extern char testneededfile[FILENAMESIZE];
#if FILEPROG
extern char progfile[FILENAMESIZE];
extern FILE *progfp;
#endif



#if FILEWARN|FILEDISP
extern char warnfile[FILENAMESIZE];
extern FILE *warnfp;
#endif
extern char warning_text[1000];

/* Define the computation phase control structures */
extern struct phase_spec *phase_hist;  /* phase_hist[p] is the spec of p.  When
                                   testing, can only use for birth features?? */
extern struct stack *deriv_order;      /* derivation_order[t] is the phases computed at
                                   t in derivation order.  site of
                                   post-prediction computation is first and may
                                   occur again later in ordering, as with
                                   adjunction newterm if exists. */
extern int *max_phase;                 /* max_phase[t] is the highest phase index used up to t */
extern int next_phase;                 /* next phase index to be used as a new phase */
extern struct stack *phase_stacks[MAXPREDBEAM];        /* stack of phases on right
                                                   frontier for each previous
                                                   prediction option REMOVE */
extern struct stack *phase_stack_scratch[MAXPREDBEAM]; /* for copying REMOVE */
extern struct stack *target_phase_stack;       /* stack of phases used by target REMOVE */
extern char **sentence_words;          /* text of words in sentence */
extern char **sentence_tagwordids;     /* text of tag and word id spec in sentence */
extern int next_site_id;               /* like next_phase but used in read_target */
extern struct stack *free_phases;      /* list of free phases used in testing */
extern int next_outind;                /* next output index to be used */

/* Define the structures for searching for the best parse during testing */
extern struct option_stack *option_queue;      /* list of options without computed phase
                                           states, sorted by probability */
extern struct option *winning_option;  /* the final best option */
extern double worst_best_option_lprob; /* the lowest log probability of the
                                   best_options */
extern struct option_stack *garbage_option_stack;       /* unallocated conss */
extern struct option *garbage_option;   /* unallocated options */
extern struct option *garbage_option_state;     /* unallocated options with
                                                   state,output arrays  */
extern int garbage_option_state_cnt;       /* count of states allocated */
#if CHECKANTIGARBAGE
extern struct option_stack *anti_garbage_options;       /* allocated options */
#endif

extern struct option *candidate_derivations;    /* structure which holds the
                                                   candidate derivations.  */
#if LOSTTAIL
extern double lost_probability;        /* used to estmate prob of unpersued options */
#endif
#if ! LOCALESTIMATE
extern double *ave_finish_probs;       /* ave_finish_probs[t] is the
                                   probability-weighted average of the 
                                   multiplication of the
                                   prediction probabilities for words t
                                   through the end. */
extern double *total_finish_probs;     /* total_finish_probs[t] is the
                                   probability-weighted total of the
                                   multiplication of the
                                   prediction probabilities for word t
                                   through the end. */
extern double *total_finish_prob_weights;      /* total_finish_prob_weights[t] is the total
                                           of the probability weights used in
                                           total_finish_probs[t].  */
#endif
/* structures which hold the target derivation.  list conses 
   allocated on need and never freed.  could do same with options.  */
extern struct option_stack *target_derivation; /* list of target options in
                                           derivation order. */
extern struct option_stack *target_derivation_next;    /* next free cons in list */
extern struct option ***best_options;  /* best_options[t][b] is the option after
                                   period t with the bth highest ranking
                                   probability, or NULL */
extern struct option_stack *training_options;  /* list of all the options which are
                                           included in target_derivation or
                                           best_options, listed in order of
                                           creation. */
extern struct option_stack *training_options_next;     /* next free cons in list */
extern struct option_stack *scratch_options;   /* for reordering other lists. */
extern struct option_stack *scratch_options_next;      /* next free cons in list */

/* Define the arrays for layer states */
extern struct input *input_hist;       /* input_hist[t] is the input at t */
extern struct hidrho **hidrho_hist;    /* (hidrho_hist[t])[p] is state at */
/* period t and phase p */
extern struct hidrho hidrho_initstate; /* used for (hidrho_hist[0])[0] */
extern struct hidrho *hidrho_constant_net;     /* hidrho_constant_net[p] memoizes net
                                           input and delta (not actually error)
                                           used for p at every period. */
extern struct hidrho hidrho_nullstate; /* zero context for new phases (nonzero not
                                   needed with INTRORHOINP=1) */
extern struct output *output_hist;     /* output_hist[p] is state at current period
                                   and phase (or index) p */
extern struct target **target_hist;    /* (target_hist[t])[p] is target output at */
/* period t and phase p */
/* could mimic output_hist to save space */


extern double default_error;     /* use for -log(0.0) (error for unnormalized
                                   values) */


/* For keeping a record of structures */
extern struct tree *target_tree;
extern struct tree *output_tree;


/* global weight state variables */
extern int weight_age;             /* the number training epochs which the current
                                   weights have gone through. (in .wgt header) */
extern double weight_pretrnerr; /* training error before/during
                                   weightfile. (in .wgt header) */


/* Define each set of weights */
/* input links (rho input = rightmost word of left-corner child, 
   tau input = most recent word) */
extern double link_r1_r[MAXNUMRHO][MAXNUMINP1];        /* rho input1 to rho hidden weight */
extern double link_r2_r[MAXNUMRHO][MAXNUMINP2];        /* rho input2 to rho hidden weight */
extern double link_t1_r[MAXNUMRHO][MAXNUMINP1];        /* tau input1 to rho hidden weight */
extern double link_t2_r[MAXNUMRHO][MAXNUMINP2];        /* tau input2 to rho hidden weight */
#if HEADWORDS
extern double link_h1_r[MAXNUMRHO][MAXNUMINP1];        /* head input1 to rho hidden weight */
extern double link_h2_r[MAXNUMRHO][MAXNUMINP2];        /* head input2 to rho hidden weight */
#endif
/* context links */
#if ! LABELCONTEXTINP
extern double link_r_r[MAXNUMRHO][MAXNUMRHO];  /* rho context to rho hidden weight */
#endif
extern double link_bir_r[MAXNUMRHO];   /* newphase input to rho hidden weight */
extern double link_tro_r[MAXNUMRHO][NUMTRIG];  /* triggering operation to rho hidden
                                           weight */
#if LABELTRIGPARENTINP
extern double link_trp_r[MAXNUMRHO][MAXNUMLAB];        /* triggering parent to rho
                                                   hidden weight */
#else
extern double link_trp_r[MAXNUMRHO][MAXNUMRHO];        /* triggering parent to rho
                                                   hidden weight */
#endif
#if LABELTRIGCHILDINP
extern double link_trc_r[MAXNUMRHO][MAXNUMLAB + MAXNUMINP1];   /* triggering child to rho
                                                           hidden weight */
#else
extern double link_trc_r[MAXNUMRHO][MAXNUMRHO];        /* triggering child to rho
                                                   hidden weight */
#endif
extern double link_lab_r[MAXNUMRHO][MAXNUMLAB + MAXNUMINP1];   /* chosen non/pre-term
                                                           label to rho hidden
                                                           weight */
extern double link_pred_r[MAXNUMRHO];  /* prev prediction to rho hidden weight */
#if CHILDCONTEXT
#if LABELCHILDINP
extern double link_chd_r[MAXNUMRHO][MAXNUMLAB + MAXNUMINP1];   /* previous child to rho
                                                           hidden weight */
#else
extern double link_chd_r[MAXNUMRHO][MAXNUMRHO];        /* previous child to rho
                                                   hidden weight */
#endif
#endif
#if HEADHIDDEN
#if LABELHEADCHILDINP
extern double link_hdc_r[MAXNUMRHO][MAXNUMLAB + MAXNUMINP1];   /* head child to rho
                                                           hidden weight */
#else
extern double link_hdc_r[MAXNUMRHO][MAXNUMRHO];        /* head child to rho
                                                   hidden weight */
#endif
#endif
#if CHOPCONTEXT
extern double link_cho_r[MAXNUMRHO][NUMCHOP];  /* previous child operation to rho
                                           hidden weight */
#endif
#if HEADOUTS
extern double link_nhd_r[MAXNUMRHO];   /* not headed to rho hidden weight */
#endif
/* output (and combination) links */
extern double link_r_o[MAXNUMOUT][MAXNUMRHO];  /* rho hidden to output weight */

/* define bias weights for each layer */
extern double bias_r[MAXNUMRHO];       /* rho hidden bias */
extern double bias_o[MAXNUMOUT];       /* output bias */


/* Define weight deltas (out[i]*del[j], no eta)       */
extern double linkdel_r1_r[MAXNUMRHO][MAXNUMINP1];     /* rho input1 to rho hidden */
extern double linkdel_r2_r[MAXNUMRHO][MAXNUMINP2];     /* rho input2 to rho hidden */
extern double linkdel_t1_r[MAXNUMRHO][MAXNUMINP1];     /* tau input1 to rho hidden */
extern double linkdel_t2_r[MAXNUMRHO][MAXNUMINP2];     /* tau input2 to rho hidden */
#if HEADWORDS
extern double linkdel_h1_r[MAXNUMRHO][MAXNUMINP1];     /* head input1 to rho hidden */
extern double linkdel_h2_r[MAXNUMRHO][MAXNUMINP2];     /* head input2 to rho hidden */
#endif
#if ! LABELCONTEXTINP
extern double linkdel_r_r[MAXNUMRHO][MAXNUMRHO];       /* rho context to rho hidden */
#endif
extern double linkdel_bir_r[MAXNUMRHO];        /* newphase input to rho hidden */
extern double linkdel_tro_r[MAXNUMRHO][NUMTRIG];       /* triggering operation to rho
                                                   hidden weight */
#if LABELTRIGPARENTINP
extern double linkdel_trp_r[MAXNUMRHO][MAXNUMLAB];     /* triggering parent to rho
                                                   hidden weight */
#else
extern double linkdel_trp_r[MAXNUMRHO][MAXNUMRHO];     /* triggering parent to rho
                                                   hidden weight */
#endif
#if LABELTRIGCHILDINP
extern double linkdel_trc_r[MAXNUMRHO][MAXNUMLAB + MAXNUMINP1];        /* triggering child to
                                                                   rho hidden weight */
#else
extern double linkdel_trc_r[MAXNUMRHO][MAXNUMRHO];     /* triggering child to rho
                                                   hidden weight */
#endif
extern double linkdel_lab_r[MAXNUMRHO][MAXNUMLAB + MAXNUMINP1];        /* chosen non/pre-term
                                                                   label to rho hidden
                                                                   weight */
extern double linkdel_pred_r[MAXNUMRHO];       /* prev prediction to rho hidden */
#if CHILDCONTEXT
#if LABELCHILDINP
extern double linkdel_chd_r[MAXNUMRHO][MAXNUMLAB + MAXNUMINP1];        /* previous child to rho
                                                                   hidden weight */
#else
extern double linkdel_chd_r[MAXNUMRHO][MAXNUMRHO];     /* previous child to rho
                                                   hidden weight */
#endif
#endif
#if HEADHIDDEN
#if LABELHEADCHILDINP
extern double linkdel_hdc_r[MAXNUMRHO][MAXNUMLAB + MAXNUMINP1];        /* head child to
                                                                   rho hidden weight */
#else
extern double linkdel_hdc_r[MAXNUMRHO][MAXNUMRHO];     /* head child to rho
                                                   hidden weight */
#endif
#endif
#if CHOPCONTEXT
extern double linkdel_cho_r[MAXNUMRHO][NUMCHOP];       /* previous child operation to rho
                                                   hidden weight */
#endif
#if HEADOUTS
extern double linkdel_nhd_r[MAXNUMRHO];        /* not headed to rho hidden weight */
#endif
extern double linkdel_r_o[MAXNUMOUT][MAXNUMRHO];       /* rho hidden to output */

/* define bias deltas */
extern double biasdel_r[MAXNUMRHO];    /* rho hidden */
extern double biasdel_o[MAXNUMOUT];    /* output */


extern int max_cl_levels;
/* Default layer sizes */
extern int numinp1;
extern int numinp2;
#define numbins  1              /* should help compiling optimization */
extern int numinp3;
extern int numrho;
extern int numout;
extern int numoutmem;
extern int numtarmem;
extern int numlabels;
extern int numout1;
extern int numpredwords;
extern int labeloutputs;    /* never reset */
#if HEADOUTS
extern int hdlabeloutputs;
extern int tagoutputs;
extern int wordoutputs;
#else
extern int tagoutputs;
extern int wordoutputs;
#endif



/** clustering of outputs for each out1 */
extern struct clustering **out_clusters_arr;


/* default input format specifications */
extern int inpwrdpos;
extern int outwrdpos;


/* default link numbers */
extern int numlink_tro;      /* never reset */
#if LABELTRIGPARENTINP
extern int numlink_trp;
#else
extern int numlink_trp;
#endif
#if LABELTRIGCHILDINP
extern int numlink_trc;
#else
extern int numlink_trc;
#endif
extern int numlink_lab;
#if CHILDCONTEXT
#if LABELCHILDINP
extern int numlink_chd;
#else
extern int numlink_chd;
#endif
#endif
#if HEADHIDDEN
#if LABELHEADCHILDINP
extern int numlink_hdc;
#else
extern int numlink_hdc;
#endif
#endif
#if CHOPCONTEXT
extern int numlink_cho;      /* never reset */
#endif

/* arrays of first word ids for each tag, and last+1 for tag+1 */
extern int inp1_first_word[MAXNUMINP1 + 1];
extern int out1_first_word[MAXNUMOUT1 + 1];
extern int out1_bin_first_word[MAXNUMOUT1][MAXNUMBINS + 1];

/* Parameters and default values */
/* learning parameters */
extern int seed;                   /* default random weight seed seed */
extern double eta;              /* default learning rate */
extern double tol;               /* default error tolerance */
extern double mom;               /* default momentum */
extern double reg;              /* default multiplicative weight decay term /eta */
extern double decay;          /* 1-reg*eta */
extern int structypecount;         /* number of structure outputs in training set */
extern double relreglabels;      /* regularization for label outpus relative to
                                   structure outputs */
extern double relregtags;        /* regularization for tag outpus relative to
                                   structure outputs */
extern double relregwords[MAXNUMOUT1]; /* regularization for word outpus relative 
                                   to structure outputs */

/* computation parameters */
extern int readcandidates;
extern int readtraincandidates;

/* learning parameter adjustment variables.  */
/* if autoreduce!=0 then eta = eta_inf*eta_red.  */
extern int autoreduce;             /* flag for automatically reducing learning rate */
extern int saveoldwgt;             /* flag for backtracking to old weight on error increase */
extern double eta_red;           /* percentage of eta_inf remaining after reduction  (in
                                   .wgt header) */
extern double eta_inf;          /* eta taken from .inf file */
extern double eta_red_rate;  /* muiltiplied by eta and added to compute
                                   eta_red during reduction */
extern int schedreduce;            /* flag for scheduled reduction of eta */
extern double sched_red_rate;        /* if used, every cycle the eta
                                           are multiplied by this rate */
/* if autoreduce_reg!=0 then reg = reg_inf-reg_red.  */
extern int autoreduce_reg;         /* flag for automatically reducing reg */
extern double reg_red;             /* subtracted from reg_inf for reduction  (in .wgt
                                   header) */
extern double reg_red_step;      /* muiltiplied by reg and added to compute
                                   reg_red during reduction */
extern double reg_inf;          /* reg taken from .inf file */
extern double finishing_prob;    /* the probability to be used as the estimate
                                   for finishing constituents on the stack. */
extern double projection_bias;   /* the amount to add to the net input of the
                                   PROJOUT output when testing */

/* testing parameters */
extern int predbeam;
extern int testing_predbeam;
extern int training_predbeam;
extern int candidate_predbeam;
extern int testing_candidate_predbeam;
extern int training_candidate_predbeam;
extern int lookahead_predbeam;
extern int lookahead;              /* 0 is flag for no lookahead pruning */
extern int phasebeam;
extern int joint_error_flag;
extern double expected_max_pred_factor;

/* output format parameters */
extern int bracketed_output;
extern int idtext_input;
extern int numtopparses;
extern int numtopderivs;
extern int stdin_word_input;
extern int stdin_candidate_input;
extern int stdout_bracketed_output;

/* global network state variables */
extern int sequence_space;             /* the sequence length before which space allocation
                                   must next occur. */
extern int state_space;                /* the number of phase states before which space allocation
                                   must next occur (not used?) */
extern int output_space;               /* the number of phases/indices before which space
                                   allocation must next occur */
extern int *period_phase_space;        /* period_phase_space[t] is the highest phase
                                   index allocated for period t */
extern int phase_space;                /* the number of phases before which space allocation must
                                   next occur for things which do not refer to period.
                                   phase_space >= max_t(period_phase_space[t]) */
extern int sentence_length;            /* the length of the last sentence to finish being
                                   input plus 1. */


/* Records of performance */
extern double sentence_prob;           /* estimated probability of current sentence */
extern double sentence_error;          /* the global error for current sentence */
extern double total_err;               /* the total error */
extern double total_local_err;         /* local computation of total_err */
extern double total_local_err_correction;      /* total_err = total_local_err + this */
extern int num_items;                  /* number of outputs   */
extern double total_targets;           /* sum of all the target values */
extern double total_struc_err, total_label_err, total_tag_err, total_word_err;
extern double total_struc_targets, total_label_targets;
extern double total_tag_targets, total_word_targets;
extern int num_sent;
extern int total_sent_correct;
extern int sent_unparsed;
extern int total_const_correct, total_const_target, total_const_output;
extern int total100_const_correct, total100_const_target, total100_const_output;
extern int total40_const_correct, total40_const_target, total40_const_output;
extern int total_const_struct_correct;
extern int total_pch_correct, total_pch_target, total_pch_output;
#if RECHECKTESTING
extern int retest_correct, retest_target, retest_output;
#endif

/* used in query_epoch */
extern int lab_correct, lab_checked;
extern int par_correct, par_target, par_output;
extern int grand_correct, grand_target, grand_output;
extern int sib_correct, sib_target, sib_output;
extern int ned_total, ned_target;
extern int ned_correct_10, ned_output_10;
extern int ned_correct_30, ned_output_30;
extern int ned_correct_50, ned_output_50;
extern int ned_correct_70, ned_output_70;
extern int ned_correct_90, ned_output_90;
extern int g_g_errs, g_s_errs, s_g_errs, s_s_errs;
extern double g_g_dist, g_s_dist, s_g_dist, s_s_dist;
#endif


